@@ -31,7 +31,7 @@ use Badger::Class
     import    => 'class',
     utils     => 'plural blessed textlike dotid camel_case',
     words     => 'ITEM ITEMS ISA',
-    constants => 'PKG ARRAY HASH REFS ONCE DEFAULT',
+    constants => 'PKG ARRAY HASH REFS ONCE DEFAULT LOADED',
     constant  => {
         OBJECT         => 'object',
         FOUND          => 'found',
@@ -85,7 +85,7 @@ sub init_factory {
     # Merge all XXXX_PATH package vars with any 'xxxx_path' or 'path' config 
     # items.  Ditto for XXXX_NAME / 'xxxx_name' / 'aka' and  XXXXS/ 'xxxxs'
     
-    my @path  = @$config{ path  => $ipath  };
+    my @path  = @$config{ path  => lc $ipath  };
     my @names = @$config{ names => lc $inames };
     $self->{ path     } = $class->list_vars(uc $ipath, @path);
     $self->{ names    } = $class->hash_vars(uc $inames, @names);
@@ -246,8 +246,7 @@ sub load {
         # what's going on in Badger::Class _autoload()
 
         my $loadname;
-        if ( ( $loadname = class($module)->maybe_load )
-        &&   ( ${ $module.PKG.VERSION } || @{ $module.PKG.ISA }  ) ) {
+        if ( ($loadname = class($module)->maybe_load) ) {
             $self->debug("loaded $module") if DEBUG;
             $loaded->{ $module } = $loadname;
             return $module 
