@@ -4,6 +4,7 @@ use Badger::Class
     version  => 3.14,
     uber     => 'Badger::Class',
     hooks    => 'wibble wobble',
+    import   => 'CLASS',
     constant => {
         CONSTANTS => 'My::Constants',
     };
@@ -18,4 +19,9 @@ sub wobble {
     $self->method( wobble => sub { "wobble: $value" } );
 }
 
+sub my_class_class {
+    CLASS
+}
+    
+
 1;
